#!/usr/bin/python
from Crypto.Cipher import AES
import os

class CbcOracle:

  BLOCKLEN = 16

  def __init__(self, iv, key):
    self.iv  = iv
    self.key = key

  # ---------------------------------------------------------
  
  def fill_with_random(self, num_bytes):
    random_bytes = os.urandom(num_bytes)

    for i in xrange(num_bytes):
      if random_bytes[i] == 0:
        random_bytes[i] = 0x01

    return random_bytes

  # ---------------------------------------------------------
  
  def pad(self, message):
    length = len(message)
    length_with_zero = length + 1
    num_pad_bytes = CbcOracle.BLOCKLEN - length_with_zero % CbcOracle.BLOCKLEN

    if num_pad_bytes == 0:
      num_pad_bytes = CbcOracle.BLOCKLEN

    padding_bytes = ["\x00" for x in xrange(num_pad_bytes + 1)]
    padding_bytes[1:] = self.fill_with_random(num_pad_bytes)
    padding_bytes = ''.join(padding_bytes)
    padded_message = message + padding_bytes
    return padded_message

    # ---------------------------------------------------------
    # An (at least to me) unreadable, but more pythonic solution could be:
    # pad = lambda s: s + (BLOCKLEN - len(s) % BLOCKLEN) * chr(BLOCKLEN - len(s) % BLOCKLEN) 
    # unpad = lambda s : s[:-ord(s[len(s)-1:])]
    # ---------------------------------------------------------

  # ---------------------------------------------------------
  
  def encrypt(self, iv, message):
    """Compute and return the CBC of the given string message."""
    cipher = AES.new(self.key, AES.MODE_CBC, iv)
    padded_message = self.pad(message)
    return cipher.encrypt(padded_message)
  
  # ---------------------------------------------------------

  def unpad(self, padded_message):
    length = len(padded_message)
    index = -1

    for i in reversed(xrange(length)):
      if padded_message[i] == '\x00':
        index = i
        break

    if index == -1:
      return None

    # print("Zero byte at {0}".format(index))
    return padded_message[:index]

  # ---------------------------------------------------------

  def was_zero_byte_in_final_block(self, clen, mlen):
    return (clen - mlen) <= CbcOracle.BLOCKLEN

  # ---------------------------------------------------------

  # def decrypt(self, iv, ciphertext):
  #   cipher = AES.new(self.key, AES.MODE_CBC, iv)
  #   padded_message = cipher.decrypt(ciphertext)
  #   message = self.unpad(padded_message)

  #   if message == None:
  #     return None

  #   if self.was_zero_byte_in_final_block(len(ciphertext), len(message)):
  #     return message

  #   return None

  # ---------------------------------------------------------

  def verify_padding(self, iv, ciphertext):
    cipher = AES.new(self.key, AES.MODE_CBC, iv)
    padded_message = cipher.decrypt(ciphertext)
    message = self.unpad(padded_message)

    if message == None:
      return False

    return self.was_zero_byte_in_final_block(
      len(ciphertext), 
      len(message)
    )
